/*
 * Arpalab.c
 *
 *  Created on: 10/11/2013
 *      Author: TomàsClar
 */

#include "Arpalab.h"

/* FreeRTOS.org includes. */
#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "task.h"
#include "type.h"
#include "uart.h"
#include "print.h"
#include "CommonUtils.h"
#include "wifly.h"


/* Demo includes. */
#include "basic_io.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

char resposta[2000];

/*****************************************************************************
** Function name:		Arpalab_send_data
**
** Descriptions:		Sends data to Arpalab server
**
** parameters:			data value (unsigned int), sensor (pointer to string)
** Returned value:		integer:	-0 unsuccessful sent
** 									-1 successful sent
**
*****************************************************************************/

unsigned int Arpalab_send_data(unsigned int data, char * sensor)
{
	unsigned int trobat;

	trobat=0;
	vTaskSuspendAll();
	{
		uint32_t i=0;
		char ip[25];
		char HTTPmsg [100];
		wifly_getIPaddress(ip);
		sprintf(HTTPmsg, "GET$/rest/stats?nid=%u&ip=%s&app=%s&data=%u",ARPA_NID, ip, sensor, data);
		printText(portCP ,"--->HTTP send\r\n");
		wifly_setHTTPclient(ARPA_SERVER, HTTPmsg);
		//clean buffer
			for(i=0; i< strlen(HTTPmsg); i++) HTTPmsg[i]='\0';

		UART0scanf(resposta, 7500);

		if((strstr(resposta, "HTTP/1.0 200 OK")) != NULL)trobat =1;

	}
	xTaskResumeAll();
	return trobat;
	}

/*****************************************************************************
** Function name:		Arpalab_request_data
**
** Descriptions:		Request data to Arpalab server
**
** parameters:			void
** Returned value:		integer: random number given by Arpalab server
**
*****************************************************************************/

unsigned int Arpalab_request_data(void)
{

	unsigned int aleatori = 0;

	vTaskSuspendAll();
	{
		uint32_t i=0;
		printText(portCP ,"--->HTTP request\r\n");
		wifly_setHTTPclient(ARPA_SERVER, "GET$/rest/feed");
		UART0scanf(resposta, 7000);
		printText(portCP, resposta);
		char * pch;
		pch = strtok(resposta, "\r\n*");
		while (pch != NULL){
			i++;
			if(i == 14)aleatori = atoi(pch);
			pch = strtok(NULL, "\r\n*");
			}
		activeDelay(250);
	}
	xTaskResumeAll();

	return aleatori;
	}
/*****************************************************************************
** Function name:		viversdomotics_request_data
**
** Descriptions:		Request data to viversdomotics server
**
** parameters:			in/out strings that contains user orders read on viversdomotics server
** Returned value:		integer:	-0 unsuccessful connection
** 									-1 successful connection
**
*****************************************************************************/
unsigned int viversdomotics_request_data(unsigned int * polv, unsigned int * calf, unsigned int * ob)
{

	unsigned int response = 0;

	vTaskSuspendAll();
	{
		uint32_t i=0;
		printText(portCP ,"--->HTTP request\r\n");
		wifly_setHTTPclient(VIVERSDOMOTICS_SERVER, "GET$/pagesprova/read_data.php");
		UART0scanf(resposta, 8000);

		if((strstr(resposta, "HTTP/1.1 200 OK")) != NULL)
		{
		response =1;

		char * pch;
		pch = strtok(resposta, "#");
		while (pch != NULL){
			i++;
			if(i == 2) *polv = atoi(pch);
			if(i == 3) *calf = atoi(pch);
			if(i == 4) *ob = atoi(pch);
			pch = strtok(NULL, "#");
			}
		}
		activeDelay(250);

	}
	xTaskResumeAll();

	return response;
	}
