/*
 * print.c
 *
 *  Created on: 14/10/2013
 *      Author: Tomàs
 */

#include "LPC17xx.h"
#include "uart.h"
#include "type.h"

#include "FreeRTOS.h"
#include "task.h"

#include <string.h>
#include <stdio.h>

extern volatile uint32_t UART0Count;
extern volatile uint8_t UART0Buffer[1024];

/*****************************************************************************
** Function name:		printInit
**
** Descriptions:		Initialize UART port, setup pin select,
**						clock, parity, stop bits, FIFO, etc.
**
** parameters:			port, baudrate
** Returned value:		None
**
*****************************************************************************/

void print_Init(uint8_t port, uint32_t baudrate)
{
	vTaskSuspendAll();
	{
		UARTInit(port,baudrate);
	}
	xTaskResumeAll();
}
/*****************************************************************************
** Function name:		printText
**
** Descriptions:		Prints a text on given port
**
** parameters:			port, pointer to a string
** Returned value:		None
**
*****************************************************************************/

void printText (uint8_t port, char *Text)
{

		UARTSend ( port, (uint8_t *)Text , strlen(Text));

}
/*****************************************************************************
** Function name:		printTextNum
**
** Descriptions:		Prints a text and a number on given port
**
** parameters:			port, pointer to a string, 32b number
** Returned value:		None
**
*****************************************************************************/

void printTextNum (uint8_t port , char *Text , uint32_t Num)
{

		char buffer[1024];
		sprintf (buffer , "%d\r\n" , Num);
		char *numero = buffer;
		int i = 0;

		while (buffer[i] != '\0') i++;

		UARTSend (port , (uint8_t *)Text , strlen ( Text) );
		UARTSend (port , ( uint8_t * )numero , i );

}

/*****************************************************************************
**                            End Of File
******************************************************************************/
