/*
 * wifly.c
 *
 *  Created on: 16/10/2013
 *      Author: Tomàs Clar
 */
#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "uart.h"
#include "type.h"
#include "wifly.h"
#include "CommonUtils.h"
#include "print.h"
#include "task.h"

#include "basic_io.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>



uint32_t sizeAsk = 0;
char resposta[1024];

/*****************************************************************************
** Function name:		wifly_Init
**
** Descriptions:		Initialize a port to work with the Wifly module
**
** parameters:			num port, baudrate
** Returned value:		None
**
*****************************************************************************/

void wifly_Init(uint8_t port, uint32_t baudrate)
{

	vTaskSuspendAll();
	{
		print_Init(port, baudrate);
		printTextNum (portCP, "UART0____ baudrate:", baudrate);
	}
	xTaskResumeAll();

	}


/*****************************************************************************
** Function name:		wifly_Reset
**
** Descriptions:		Sends a sequence 1-0-1 to pin 24 (connected to reset pin -5- in wifly module).
** 						This causes the reset of the module.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void wifly_Reset(void)
{
	vTaskSuspendAll();
	{
		LPC_PINCON->PINSEL1 &=(~(3<<12));
		LPC_GPIO0->FIODIR |= (1<< RST_WIFLY_PIN);
		LPC_GPIO0->FIOCLR = 1 << RST_WIFLY_PIN;
		activeDelay(5000);
		LPC_GPIO0->FIOSET = 1 << RST_WIFLY_PIN;
	}
	xTaskResumeAll();
	}


/*****************************************************************************
** Function name:		wifly_waitForResponse
**
** Descriptions:		Used for study the response to an order. For debug mode
**
** parameters:			pointer to string "ordre", num bytes of response
** Returned value:		None
**
*****************************************************************************/

void wifly_waitForResponse( char * ordre, uint32_t nbytes)
{
	vTaskSuspendAll();
	{
		sizeAsk=UART0scanf(resposta, 4000);
		printText(portCP, "WiFly response to---> ");
		printText(portCP, ordre);
		printText(portCP, "\r\n");
		printText(portCP, resposta);
		nbytes=sizeAsk;
	}
	xTaskResumeAll();
	}

/*****************************************************************************
** Function name:		wiflyCommandMode_request
**
** Descriptions:		Sends the string $$$ to the wifly module
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/

static void wiflyCommandMode_request(void)
{
	printText(portWifly, "$$$");
	sizeAsk=UART0scanf(resposta, 2000);
	}
/*****************************************************************************
** Function name:		wiflyCommandMode_exit
**
** Descriptions:		Returns the wifly module to the data mode
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
static void wiflyCommandMode_exit(void)
{
	printText(portWifly, "exit\r\n");

	}
/*****************************************************************************
** Function name:		wifly_wifiConnect
**
** Descriptions:		Associates to an AP using the given parameters. Shows the sequence on
** 						portCP.
**
** parameters:			ssid, passwd, channel, authenticacion mode (0-none, 1-WEP, 2-WPA1
** 						3-mixed WPA1 WPA2, 4-WPA2 PSK)
** Returned value:		None
**
*****************************************************************************/
/**/
void wifly_wifiConnect(uint8_t * ssid, uint8_t * phrase, uint8_t channel, uint8_t auth)
{

		char buffer[100];

		wiflyCommandMode_request();

		printText(portCP,"\r--->Command set wlan auth\r\n");
		sprintf (buffer , "set wlan auth %u\r\n" , auth);
		printText(portWifly, buffer);

		activeDelay(800);


		printText(portCP,"\r--->Command set wlan ssid\r\n");
		sprintf (buffer , "set wlan ssid %s\r\n" , ssid);
		printText(portWifly, buffer);

		activeDelay(800);

		printText(portCP,"\r--->Command set wlan phrase\r\n");
		sprintf (buffer, "set wlan phrase %s\r\n", phrase);
		printText(portWifly, buffer);

		activeDelay(800);

		printText(portCP,"\r--->Command set wlan channel\r\n");
		sprintf (buffer, "set wlan channel %u\r\n", channel);
		printText(portWifly, buffer);

		activeDelay(800);

		printText(portCP, "\r--->Comand exit....waiting for wifi association\r\n");
		wiflyCommandMode_exit();
		/*associating*/
		printText(portCP, "--->Printing association details\r\n");
		sizeAsk=UART0scanf(resposta, 6000);
		printText(portCP, resposta);

	}

/*****************************************************************************
** Function name:		wifly_pingTestServer
**
** Descriptions:		Sends the num of pings expressed by paramenter "pings" to a server.
** 						Shows the response on portCP
**
** parameters:			Test-server, num of pings
** Returned value:		None
**
*****************************************************************************/
void wifly_pingTestServer(uint8_t * testserver, uint8_t pings)
{
	vTaskSuspendAll();
	{
		char buffer[100];

		wiflyCommandMode_request();
		printText(portCP, "\r--->Command ping to TESTSERVER\r\n");
		sprintf (buffer, "ping %s %u\r\n", testserver, pings);
		printText(portWifly, buffer);

		sizeAsk=UART0scanf(resposta, 5000);
		printText(portCP, resposta);
		wiflyCommandMode_exit();
	}
	xTaskResumeAll();
	}
/*****************************************************************************
** Function name:		wifly_getIPsettings
**
** Descriptions:		Shows the IP settings of the Wifly module on portCP
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void wifly_getIPsettings(void)
{
	vTaskSuspendAll();
	{
		wiflyCommandMode_request();
		printText(portCP, "\r--->Command get ip\r\n");
		printText(portWifly, "get ip\r\n");

		sizeAsk=UART0scanf(resposta, 2000);
		printText(portCP, resposta);
		wiflyCommandMode_exit();
	}
	xTaskResumeAll();
	}
/*****************************************************************************
** Function name:		wifly_getWlanSettings
**
** Descriptions:		Shows ssid, channel and other wlan settings on portCP
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void wifly_getWlanSettings(void)
{
	vTaskSuspendAll();
	{
		wiflyCommandMode_request();
		printText(portCP, "\r--->Command get wlan\r\n");
		printText(portWifly, "get wlan\r\n");
		sizeAsk=UART0scanf(resposta, 2000);
		printText(portCP, resposta);
		wiflyCommandMode_exit();
	}
	xTaskResumeAll();
	}
/*****************************************************************************
** Function name:		wifly_getRSSI
**
** Descriptions:		Gets the last received signal strength
**
** parameters:			None
** Returned value:		RSSI
**
*****************************************************************************/

uint32_t wifly_getRSSI(void)
{
	uint32_t rssivalue=0;
	vTaskSuspendAll();
	{

		wiflyCommandMode_request();
		printText(portWifly, "show rssi\r\n");
		sizeAsk=UART0scanf(resposta, 2000);

		char *pch;
		uint8_t i=0;
		pch = strtok(resposta, "-)");
		while(pch != NULL){
			i++;
			if(i==2) rssivalue=atoi(pch);
			pch=strtok(NULL,"-)");
		}

		wiflyCommandMode_exit();
	}
	xTaskResumeAll();

	return rssivalue;

	}

/*****************************************************************************
** Function name:		wifly_getTime
**
** Descriptions:		Gets the number of seconds since last power up or reboot of the wifly module
**
** parameters:			None
** Returned value:		seconds since last power up
**
*****************************************************************************/

uint32_t wifly_getTime(void)
{
	uint32_t seconds = 0;
	vTaskSuspendAll();
	{

		wiflyCommandMode_request();
		printText(portWifly, "show time\r\n");
		sizeAsk=UART0scanf(resposta, 2000);
		activeDelay(500);
		char *pch;
		uint8_t i=0;
		pch=strtok(resposta, "= ");
		while(pch != NULL){
			i++;
			if(i==5) seconds= atoi(pch);
			pch = strtok(NULL, "= ");
		}
		wiflyCommandMode_exit();
	}
	xTaskResumeAll();

	return seconds;

	}
/*****************************************************************************
** Function name:		wifly_setWiFi_TxPower
**
** Descriptions:		Sets the Wi-Fi transmit power
**
** parameters:			new Tx power (from 1 to 12dBm. 0 corresponds to 12)
** Returned value:		None
**
*****************************************************************************/
void wifly_setWiFi_TxPower(uint8_t txpower)
{
	vTaskSuspendAll();
	{
		char buffer[100];

		wiflyCommandMode_request();
		printText(portCP, "\r--->Command set wlan tx\r\n");
		sprintf (buffer, "set wlan tx %u\r\n", txpower);
		printText(portWifly, buffer);

		sizeAsk=UART0scanf(resposta, 2000);
		printText(portCP, resposta);
		wiflyCommandMode_exit();
	}
	xTaskResumeAll();
	}
/*****************************************************************************
** Function name:		wifly_getIPaddress
**
** Descriptions:		Gets a string that represents the IP address of the wifly module
**
** parameters:			pointer to a string IP (in-out parameter)
** Returned value:		None
**
*****************************************************************************/
void wifly_getIPaddress(char *ipaddress)
{
	vTaskSuspendAll();
	{

		char *pch;
		uint8_t cont = 0;
		wiflyCommandMode_request();
		printText(portWifly, "get ip a\r\n");
		sizeAsk=UART0scanf(resposta, 4000);

		pch = strtok (resposta, "\r\n");
		while (pch != NULL){
			cont ++;
			if(cont == 2) sprintf(ipaddress, "%s", pch);
			pch = strtok (NULL, "\r\n");
		}
		wiflyCommandMode_exit();
	}
	xTaskResumeAll();
}
/*****************************************************************************
** Function name:		wifly_setHTTPclient
**
** Descriptions:		Prepares wifly module to send data to a server via HTTP
**
** parameters:			Server, HTTP message
** Returned value:		void
**
*****************************************************************************/

void wifly_setHTTPclient(char * server, char * httpmsg)
{
	vTaskSuspendAll();
	{
	uint32_t i=0;
	char buffer[150];
	wiflyCommandMode_request();
	printText (portWifly, "set ip proto 18\r\n");
	activeDelay(500);
	sprintf(buffer, "set dns name %s\r\n", server);
	printText(portWifly, buffer);
	//clean buffer
		for(i=0; i< strlen(buffer); i++) buffer[i]='\0';

	activeDelay(400);
	printText (portWifly, "set ip host 0\r\n");
	activeDelay(1000);
	printText (portWifly, "set ip remote 80\r\n");
	activeDelay(500);
	printText (portWifly, "set option format 1\r\n");
	activeDelay(500);
	sprintf(buffer, "set com remote %s\n\r", httpmsg);
	printText (portWifly, buffer);
	//clean buffer
		for(i=0; i< strlen(buffer); i++) buffer[i]='\0';

	activeDelay(400);
	printText (portWifly, "open\r\n\r\n");
	}
	xTaskResumeAll();


	}



/*****************************************************************************
** Function name:		wifly_tcp_connect
**
** Descriptions:		Open a tcp connection
**
** parameters:			host, port)
** Returned value:		None
**
*****************************************************************************/
void wifly_tcp_connect(uint8_t * host, uint32_t port)
{
	uint8_t i = 0;
	char buffer[150];

	wiflyCommandMode_request();
	printText (portWifly, "set ip proto 8\r\n");
	activeDelay(500);
	sprintf(buffer, "set ip remote %u\r\n", port);
	printText (portWifly, buffer);
	activeDelay(500);
	//clean buffer
			for(i=0; i< strlen(buffer); i++) buffer[i]='\0';
	sprintf(buffer, "set ip host %s\r\n", host);
	printText(portWifly, buffer);
	activeDelay(500);
	//clean buffer
			for(i=0; i< strlen(buffer); i++) buffer[i]='\0';
	printText(portWifly, "set com remote TCP$CONNECT\r\n");
	activeDelay(500);
	printText(portWifly, "open\r\n");
	activeDelay(500);

	}
/*****************************************************************************
** Function name:		wifly_setBaudrate
**
** Descriptions:		Changes the wifly baudrate
**
** parameters:			New baudrate
** Returned value:		None
**
*****************************************************************************/
void wifly_setBaudrate(uint32_t newbaudrate)
{
	uint8_t i =0;
	char buffer[100];
	if((newbaudrate >= 2400)||(newbaudrate >= 921600))
	wiflyCommandMode_request();
	activeDelay(500);
	sprintf(buffer,"set uart baud %s\r\n", newbaudrate);
	printText(portWifly, buffer);
	//clean buffer
		for(i=0; i< strlen(buffer); i++) buffer[i]='\0';
	wiflyCommandMode_exit();
	print_Init(portWifly, newbaudrate);

	}


/*****************************************************************************
**                            End Of File
******************************************************************************/
