/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "queue.h"
#include "type.h"
#include "uart.h"
#include "print.h"
#include "CommonUtils.h"
#include "Arpalab.h"
#include "adc.h"
#include "pwm.h"
#include "dht.h"



/* Demo includes. */
#include "basic_io.h"
#include <string.h>
#include <stdio.h>



#define  portCP 					(0x3)
#define  portWifly  				(0x0)
#define CHANNEL_NUM					(0X1)
/* The task functions. */
void vTask1( void *pvParameters );
void vTask2( void *pvParameters );
void vTask3( void *pvParameters );
void vTask4( void *pvParameters );

uint32_t baudrate = 9600;
uint32_t numbytes=0;
extern volatile uint32_t ADCValue;
uint32_t ref, Kobert;

//Wifi parameters
uint8_t * SSID = "FTE-E08E";
uint8_t * PHRASE = "marineta2010";
uint8_t CHANNEL = 1;
uint8_t AUTH = 1;
uint8_t IP [25];
//test parameters
uint8_t * TESTSERVER = "www.arpalab.appspot.com";
uint8_t DEFAULT_PINGS = 1;
uint8_t FAIL_COUNTER;


uint32_t NID=514718;
uint32_t ORDER_INTERVAL = 5000; //collect order frequency in ms
uint32_t SENSOR_INTERVAL = 120000; //collect sensor data frequency in ms


volatile int generalCounter, maxobert, minobert;


xSemaphoreHandle xMutex;
xTaskHandle xTask1Handle, xTask2Handle, xTask3Handle, xTask4Handle;


uint8_t resposta[1024];
uint32_t aleatori, rssivalue, j;

/*-----------------------------------------------------------*/

int main( void )
{

	uint8_t humtest, temptest;

	/* Init the UART port3 9600 baudrate. */
	print_Init(portCP, baudrate);
	printTextNum(portCP, "UART3____ baudrate:", baudrate);


	/* Init the UART port0 9600 baudrate. */
	wifly_Init(portWifly, baudrate);
	printTextNum(portCP, "Initiating portWifly ", baudrate);
	/*Waiting for a correct Wifly module switch on*/
	activeDelay(3000);

	/*Wifly reset. LPC1769Board -> 0-1-0 in pin 24 */
	wifly_Reset();

	/*Take and print wifly response*/
	wifly_waitForResponse("RESET", numbytes);

	/*FUNCTIONALITY TEST*/
	//Associates to an AP
	FAIL_COUNTER=0;
	printText(portCP, "ASSOCIATING TO AN AP\r\n");
	wifly_wifiConnect(SSID, PHRASE, CHANNEL, AUTH);

	wifly_getIPaddress(IP);

	//Ping test to a server
	wifly_pingTestServer (TESTSERVER, DEFAULT_PINGS);

	printText(portCP, "Booting actuators & sensors\r\n");

	//Initializes and test pins P0.17-P0.18
	pin_init((15<<2), (3<<17));
	printText(portCP, "Testing outputs \r\n");
	pin_on(3<<17);
	activeDelay(500);
	pin_off(3<<17);

	//Initialitzes DHT temperature sensors, pins P2.6 ~ P2.8
	dht_init();

	//Initializes ADC
	ADCInit(ADC_CLK);

	//Initializes and test PWM P2.0
	//Takes the ADC value to autoadjust
	PWM_Init(CHANNEL_NUM, 0);
	PWM_Set(CHANNEL_NUM, PWM_CYCLE, 21300);
	PWM_Start(CHANNEL_NUM);
	activeDelay(1000);
	maxobert = ADCRead(0);
	PWM_Stop(CHANNEL_NUM);
	PWM_Set (CHANNEL_NUM, PWM_CYCLE, 0);
	PWM_Start (CHANNEL_NUM);
	activeDelay(1000);
	minobert = ADCRead(0);

	xMutex=xSemaphoreCreateMutex();

	if(xMutex != NULL)

	{


	/* Create one of the two tasks. */
	xTaskCreate(	vTask1,		/* Pointer to the function that implements the task. */
					"Task 1",	/* Text name for the task.  This is to facilitate debugging only. */
					1000,		/* Stack depth in words. */
					NULL,		/* We are not using the task parameter. */
					2,			/* This task will run at priority 2. */
					&xTask1Handle );		/* We are using the task handle. */


	/* Create the other task in exactly the same way. */
	xTaskCreate( vTask2, "Task 2", 1000, NULL, 3, &xTask2Handle);
	xTaskCreate( vTask3, "Task 3", 1000, NULL, 1, &xTask3Handle);
	xTaskCreate( vTask4, "Task 4", 1000, NULL, 1, &xTask4Handle);
	/* Start the scheduler so our tasks start executing. */
	vTaskStartScheduler();

	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */

	}
	else
	{
		printText(portCP, "WARNING! cannot create semaphore\r\n");
	}
	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/

void vTask1( void *pvParameters )
{
const char *pcTaskName = "Task 1 ACTUADORS\r\n";
portBASE_TYPE xStatus;
portTickType xLastWakeTime;
uint16_t resposta;
uint8_t i = 0;
unsigned int polv=0, calf=0, obert=0, obert_ini=0;
unsigned portBASE_TYPE prioritat;




	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		//this instruction can be placed outside of the "for" clause if we want a periodic cycle
		xLastWakeTime = xTaskGetTickCount();
		xSemaphoreTake(xMutex, portMAX_DELAY);
			{


				printText(portCP, (uint8_t *)pcTaskName);
				prioritat=uxTaskPriorityGet(NULL);

				resposta = 0;
				//we'll try for 5 times
				do{
					i++;
					resposta = viversdomotics_request_data(&polv, &calf, &obert);
				} while ((resposta == 0)&&(i <= 5));

				i=0;
				if(resposta == 0)
			{
			//we'll wait for the next turn
			printText(portCP, "ERROR Test server unavailable\r\n");
			FAIL_COUNTER++;
			if(FAIL_COUNTER > 5 )
			{
				printText(portCP, "wlan seems down...reset in 20 sec\r\n");
				vTaskDelay(20000 / portTICK_RATE_MS);
				wifly_Init(portWifly, baudrate);
				printText(portCP, "Port 0 initiated ... active delay (7 sec)\r\n");
				/*Waiting for a correct Wifly module switch on*/
				activeDelay(3000);
				wifly_Reset();
				printText(portCP, "ASSOCIATING TO AN AP\r\n");
				wifly_wifiConnect(SSID, PHRASE, CHANNEL, AUTH);
				FAIL_COUNTER=0;
				}
			}
			else
			{
				printTextNum(portCP, "polv = ", polv);
				printTextNum(portCP, "calf = ", calf);
				printTextNum(portCP, "obert = ", obert);
				//Enable or disable FAN-WATER
				if (polv == 1)
				{
					pin_on(1<<17);
				}
				else
				{
					pin_off(1<<17);
				}
				if (calf == 1)
				{
					pin_on(1<<18);
				}
				else
				{
					pin_off(1<<18);
				}
				//Enable or disable servo
				if (obert != obert_ini)
				{
					obert_ini=obert;
					PWM_Stop(CHANNEL_NUM);

					switch(obert)
					{
					case 0:
					default:
						PWM_Set(CHANNEL_NUM, PWM_CYCLE, 0);
					break;
					case 25:
						PWM_Set(CHANNEL_NUM, PWM_CYCLE, 5330);
					break;
					case 50:
						PWM_Set(CHANNEL_NUM, PWM_CYCLE, 10650);
					break;
					case 75:
						PWM_Set(CHANNEL_NUM, PWM_CYCLE, 15900);
					break;
					case 100:
						PWM_Set(CHANNEL_NUM, PWM_CYCLE, 21300);
					break;
					}
					PWM_Start (CHANNEL_NUM);
				}

			}

				vTaskPrioritySet(NULL, 2);
				vTaskDelayUntil(&xLastWakeTime, (ORDER_INTERVAL/portTICK_RATE_MS));
		}
		xSemaphoreGive(xMutex);
		taskYIELD();
	}

}
/*-----------------------------------------------------------*/
void vTask2( void *pvParameters )
{
const char *pcTaskName = "Task 2 SENSOR FINESTRA\r\n";
int obertura = 0;
uint8_t connect, i;
portTickType xLastWakeTime;
unsigned portBASE_TYPE prioritat;



	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		//this instruction can be placed outside of the "for" clause if we want a periodic cycle
		xLastWakeTime = xTaskGetTickCount();
		vTaskDelayUntil (&xLastWakeTime, (SENSOR_INTERVAL/ portTICK_RATE_MS));
		connect=0;
		i=0;

		xSemaphoreTake(xMutex, portMAX_DELAY);
		{
			printText(portCP, (uint8_t *)pcTaskName);
			prioritat=uxTaskPriorityGet(NULL);


			ADCValue=ADCRead(0);

			//conversion from the accelerometer value to a %
			obertura = (100 * (minobert-ADCValue))/(minobert-maxobert);
			if (ADCValue > minobert) obertura = 0;

			printTextNum(portCP, "Finestra oberta % : ", obertura);

			//'connect' informs of correct or incorrect sending. we'll try for 5 times
			do{
			i++;
			connect=Arpalab_send_data(obertura,"obrt");

			}while((connect==0)&&(i<= 5));
			if(connect==0)
			{
				//we'll wait for the next turn
				printText(portCP, "ERROR Test server unavailable\r\n");
				FAIL_COUNTER++;
				if(FAIL_COUNTER > 5 )
					{
						printText(portCP, "wlan seems down...reset in 20 sec\r\n");

						vTaskDelay(20000 / portTICK_RATE_MS);
						wifly_Init(portWifly, baudrate);
						printText(portCP, "Port 0 initiated ... active delay (7 sec)\r\n");
						/*Waiting for a correct Wifly module switch on*/
						activeDelay(3000);
						wifly_Reset();
						printText(portCP, "ASSOCIATING TO AN AP\r\n");
						wifly_wifiConnect(SSID, PHRASE, CHANNEL, AUTH);
						FAIL_COUNTER=0;
					}
				}
				else
				{
					printTextNum(portCP, "obrt sent at time: ", i);
				}
			vTaskPrioritySet(NULL, 1);
			vTaskPrioritySet(xTask3Handle, 3);

			}
			xSemaphoreGive(xMutex);
			taskYIELD();
	}

}

/*-----------------------------------------------------------*/
void vTask3( void *pvParameters )
{
const char *pcTaskName = "Task 3 SENSORS INTERIORS\r\n";
uint8_t connect, i, HR_int = 0, Temp_int=0, humTerra=0;
unsigned portBASE_TYPE prioritat;


	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		connect=0;
		i=0;
		xSemaphoreTake(xMutex, portMAX_DELAY);
		{
			printText(portCP, (uint8_t *)pcTaskName);
			prioritat=uxTaskPriorityGet(NULL);


			//read temperature sensor
			dht_request(6, &HR_int, &Temp_int);
			//read soil humidity
			ADCValue=ADCRead(1);
			humTerra=ADCValue/50;

			//PROVES
			printTextNum(portCP, "HRinterior: ", HR_int);
			printTextNum(portCP, "TempInterior: ", Temp_int);
			printTextNum(portCP, "HumitatTerra: ", humTerra);

			//'connect' informs of correct or incorrect sending. we'll try for 5 times
			do{
			i++;
			connect=Arpalab_send_data(HR_int,"hrin");

			}while((connect==0)&&(i<= 5));
			if(connect==0)
			{
				//we'll wait for the next turn
				printText(portCP, "ERROR Test server unavailable\r\n");
				FAIL_COUNTER++;
				if(FAIL_COUNTER > 5 )
					{
						printText(portCP, "wlan seems down...reset in 20 sec\r\n");

						vTaskDelay(20000 / portTICK_RATE_MS);
						wifly_Init(portWifly, baudrate);
						printText(portCP, "Port 0 initiated ... active delay (7 sec)\r\n");
						/*Waiting for a correct Wifly module switch on*/
						activeDelay(3000);
						wifly_Reset();
						printText(portCP, "ASSOCIATING TO AN AP\r\n");
						wifly_wifiConnect(SSID, PHRASE, CHANNEL, AUTH);
						FAIL_COUNTER=0;
					}
				}
				else
				{
					printTextNum(portCP, "hrin sent at time: ", i);
				}
			i=0;
			do{
			i++;
			connect=Arpalab_send_data(Temp_int,"tein");

			}while((connect==0)&&(i<= 5));
			if(connect==0)
			{
				//we'll wait for the next turn
				printText(portCP, "ERROR Test server unavailable\r\n");
				FAIL_COUNTER++;
				if(FAIL_COUNTER > 5 )
					{
						printText(portCP, "wlan seems down...reset in 20 sec\r\n");

						vTaskDelay(20000 / portTICK_RATE_MS);
						wifly_Init(portWifly, baudrate);
						printText(portCP, "Port 0 initiated ... active delay (7 sec)\r\n");
						/*Waiting for a correct Wifly module switch on*/
						activeDelay(3000);
						wifly_Reset();
						printText(portCP, "ASSOCIATING TO AN AP\r\n");
						wifly_wifiConnect(SSID, PHRASE, CHANNEL, AUTH);
						FAIL_COUNTER=0;
					}
				}
				else
				{
					printTextNum(portCP, "tein sent at time: ", i);
				}

			i=0;
			do{
			i++;
			connect=Arpalab_send_data(humTerra,"hter");

			}while((connect==0)&&(i<= 5));
			if(connect==0)
			{
				//we'll wait for the next turn
				printText(portCP, "ERROR Test server unavailable\r\n");
				FAIL_COUNTER++;
				if(FAIL_COUNTER > 5 )
					{
						printText(portCP, "wlan seems down...reset in 20 sec\r\n");

						vTaskDelay(20000 / portTICK_RATE_MS);
						wifly_Init(portWifly, baudrate);
						printText(portCP, "Port 0 initiated ... active delay (7 sec)\r\n");
						/*Waiting for a correct Wifly module switch on*/
						activeDelay(3000);
						wifly_Reset();
						printText(portCP, "ASSOCIATING TO AN AP\r\n");
						wifly_wifiConnect(SSID, PHRASE, CHANNEL, AUTH);
						FAIL_COUNTER=0;
					}
				}
				else
				{
					printTextNum(portCP, "hter sent at time: ", i);
				}

			vTaskPrioritySet (xTask4Handle, 3);
			vTaskPrioritySet(NULL, 1);

			}
			xSemaphoreGive(xMutex);
			taskYIELD();
	}

}
/*-----------------------------------------------------------*/
void vTask4( void *pvParameters )
{
const char *pcTaskName = "Task 4 SENSORS EXTERIORS\r\n";
uint8_t connect, i, HR_ext = 0, Temp_ext=0;
unsigned portBASE_TYPE prioritat;



	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		connect=0;
		i=0;
		xSemaphoreTake(xMutex, portMAX_DELAY);
		{

		printText(portCP, (uint8_t *)pcTaskName);
		prioritat=uxTaskPriorityGet(NULL);

		//read temperature sensor
		dht_request(7, &HR_ext, &Temp_ext);

		//PROVES
		printTextNum(portCP, "HRexterior: ", HR_ext);
		printTextNum(portCP, "TempExterior: ", Temp_ext);

		//'connect' informs of correct or incorrect sending. we'll try for 5 times
		do{
		i++;
		connect=Arpalab_send_data(HR_ext,"hrex");

		}while((connect==0)&&(i<= 5));
		if(connect==0)
		{
			//we'll wait for the next turn
			printText(portCP, "ERROR Test server unavailable\r\n");
			FAIL_COUNTER++;
			if(FAIL_COUNTER > 5 )
				{
					printText(portCP, "wlan seems down...reset in 20 sec\r\n");

					vTaskDelay(20000 / portTICK_RATE_MS);
					wifly_Init(portWifly, baudrate);
					printText(portCP, "Port 0 initiated ... active delay (7 sec)\r\n");
					/*Waiting for a correct Wifly module switch on*/
					activeDelay(3000);
					wifly_Reset();
					printText(portCP, "ASSOCIATING TO AN AP\r\n");
					wifly_wifiConnect(SSID, PHRASE, CHANNEL, AUTH);
					FAIL_COUNTER=0;
				}
			}
			else
			{
				printTextNum(portCP, "hrext sent at time: ", i);
			}
		i=0;
		do{
		i++;
		connect=Arpalab_send_data(Temp_ext,"teex");

		}while((connect==0)&&(i<= 5));
		if(connect==0)
		{
			//we'll wait for the next turn
			printText(portCP, "ERROR Test server unavailable\r\n");
			FAIL_COUNTER++;
			if(FAIL_COUNTER > 5 )
				{
					printText(portCP, "wlan seems down...reset in 20 sec\r\n");

					vTaskDelay(20000 / portTICK_RATE_MS);
					wifly_Init(portWifly, baudrate);
					printText(portCP, "Port 0 initiated ... active delay (7 sec)\r\n");
					/*Waiting for a correct Wifly module switch on*/
					activeDelay(3000);
					wifly_Reset();
					printText(portCP, "ASSOCIATING TO AN AP\r\n");
					wifly_wifiConnect(SSID, PHRASE, CHANNEL, AUTH);
					FAIL_COUNTER=0;
				}
			}
			else
			{
				printTextNum(portCP, "te_ext sent at time: ", i);
			}




		vTaskPrioritySet(NULL, 1);
		vTaskPrioritySet(xTask2Handle, 3);
		}
		xSemaphoreGive(xMutex);
		taskYIELD();

	}
}



/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}


